from polygon import *
from turtle import *

class LSystem:
    def __init__(self):
        self.word = None
        self.rules = None
    def iterate(self):
        new_word = []
        for i, w in enumerate(self.word):
            aux = self.rules(self.word, i)
            for a in aux:
                new_word.append(a)
        self.word = new_word
    def to_vec_instr(self):
        raise NotImplementedError('call to abstract conversion method')


# Example - Barnesley fern
class BarnFern(LSystem):
    def __init__(self):
        self.word = ['X']
        def rules(word, i):
            dic_rules = {
                'X': list('F+[[X]-X]-F[-FX]+X'),
                'F': list('FF'),
                '+' : '+',
                '-' : '-',
                '[':'[', 
                ']':']'}
            return dic_rules[word[i]]
        self.rules = rules
    def to_vec_instr(self):
        instr = []
        switcher = {
            'F':Df(1),
            '+':Rot(mathutils.Quaternion((0, 0, 1), math.radians(25))), #left 25
            '-':Rot(mathutils.Quaternion((0, 0, 1), math.radians(-25))),
            '[':Push(),
            ']':Pop(),
        }
        for c in self.word:
            if switcher.get(c, None) is not None:
                instr.append(switcher[c])
        return instr
           
            
        

bf = BarnFern()

for i in range(6):
    bf.iterate()
#print(bf.word)
#print(bf.to_vec_instr())


orient = mathutils.Quaternion((1,0,0,0)) #identity quaternion
orient = mathutils.Quaternion((0, 1, 0), math.radians(-90))
pos = Vector([0, 0, 0])
turtle = PTurtle(pos, orient, 0)
stack = []
mesh = Mesh([Vector([0,0,0])], [])

Instruction.exec_all(turtle, stack, mesh, bf.to_vec_instr())


mesh.add_to_scene("mesh barnFern", "obj barnFern")
bpy.ops.wm.save_mainfile()



        
