import mathutils
import math
from mathutils import Vector
from turtle import Turtle, Instruction

import bpy
from bpy.types import Operator
from bpy.props import FloatVectorProperty
from bpy_extras.object_utils import AddObjectHelper, object_data_add

class Mesh:
    def add_to_scene(self, mesh_name, obj_name):
        # clean this method
        mesh = bpy.data.meshes.new(name=mesh_name)
        mesh.from_pydata(self.v, self.e, [])
        theObj = bpy.data.objects.new(obj_name, mesh)
        bpy.context.collection.objects.link(theObj)
        blender_ob = bpy.context.collection.objects[obj_name]
        blender_ob.select_set(state=True)
        bpy.context.view_layer.objects.active = blender_ob
        bpy.ops.object.convert(target='CURVE')
        bpy.context.object.data.bevel_depth = 0.1 
    def __str__(self):
        out = "\nPolygon mesh:\n"
        out+= "   v = "+str(self.v)+"\n"
        out+= "   e = "+str(self.e)+"\n"
        return out
    def __init__(self, v, e):
        self.v = v
        self.e = e

class PTurtle(Turtle):
    def clone(self):
        return PTurtle(self.pos, self.orient, self.v_t)
    def __init__(self, pos, orient, v_t):
        super().__init__(pos, orient)
        self.v_t = v_t
    def follow(self, pt):
        super().follow(pt)
        self.v_t = pt.v_t

class Rot(Instruction):
    def __init__(self, rot):
        self.rot = rot
    def exec(self, pturtle, stack, pmesh):
        pturtle.rot(self.rot)

class Wf(Instruction):
    def __init__(self, dist):
        self.dist = dist
    def exec(self, pturtle, stack, mesh):
        pturtle.wf(self.dist)
        pturtle.v_t = len(mesh.v)
        mesh.v.append(pturtle.pos)

class Df(Instruction):
    def __init__(self, dist):
        self.dist = dist
    def exec(self, pturtle, stack, mesh):
        pturtle.wf(self.dist)
        mesh.v.append(pturtle.pos)
        nv = len(mesh.v)
        mesh.e.append([pturtle.v_t, nv-1])
        pturtle.v_t = nv-1

class Push(Instruction):
    def exec(self, pturtle, stack, mesh):
        stack.append(pturtle.clone())

class Pop(Instruction):
    def exec(self, pturtle, stack, mesh):
        pturtle.follow(stack.pop())
        # blender dk how to draw properly, then we do this hack
        mesh.v.append(pturtle.pos)
        pturtle.v_t = len(mesh.v)-1

